﻿Imports System.IO
Imports System.Xml
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Documents
Imports System.Text.RegularExpressions

<TemplatePart(Name:="ItemTextBlock", Type:=GetType(TextBlock))>
    Public Class ItineraryItemBlock
    Inherits Control
    ''' <summary>
    ''' The textblock that will contain the text.
    ''' </summary>
    Private textBlock As TextBlock

    ''' <summary>
    ''' The text.
    ''' </summary>
    Public Property Text As String
        Get
            Return CStr(GetValue(TextProperty))
        End Get
        Set(ByVal value As String)
            SetValue(TextProperty, value)
        End Set
    End Property

    ''' <summary>
    ''' Dependency property for text.
    ''' </summary>
    Public Shared ReadOnly TextProperty As DependencyProperty =
        DependencyProperty.Register("Text", GetType(String),
                                    GetType(ItineraryItemBlock),
                                    New PropertyMetadata("", AddressOf OnTextChanged))

    ''' <summary>
    ''' When the text changes, update the content of the control.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Shared Sub OnTextChanged(ByVal sender As Object,
                                     ByVal e As DependencyPropertyChangedEventArgs)
        Dim item = TryCast(sender, ItineraryItemBlock)
        If item IsNot Nothing Then
            item.SetItinerary(TryCast(e.NewValue, String))
        End If
    End Sub

    ''' <summary>
    ''' When the template is applied, find the textblock.
    ''' </summary>
    Public Overrides Sub OnApplyTemplate()
        MyBase.OnApplyTemplate()

        ' Get textblock.
        textBlock = TryCast(GetTemplateChild("ItemTextBlock"), TextBlock)
        If textBlock Is Nothing Then
            Throw New InvalidOperationException("Unable to find 'ItemTextBlock' TextBlock in the template.")
        End If

        ' Set the text if it was assigned before loading the template.
        If Not String.IsNullOrEmpty(Text) Then
            SetItinerary(Text)
        End If
    End Sub

    ''' <summary>
    ''' Cleans the unnecessary elements from the itinerary.
    ''' Reason is that TurnDir doesn't always close correctly and this causes a bug in the XmlReader.
    ''' </summary>
    ''' <param name="itinerary">The itinerary.</param>
    ''' <returns></returns>
    Private Function CleanElements(ByVal itinerary As String) As String
        If itinerary IsNot Nothing Then
            itinerary = itinerary.Replace("<VirtualEarth:Action>", String.Empty)
            itinerary = itinerary.Replace("</VirtualEarth:Action>", String.Empty)
            itinerary = itinerary.Replace("<VirtualEarth:TurnDir>", String.Empty)
            itinerary = itinerary.Replace("</VirtualEarth:TurnDir>", String.Empty)
        End If

        ' Done.
        Return itinerary
    End Function

    ''' <summary>
    ''' Sets the itinerary.
    ''' </summary>
    ''' <param name="text">The text.</param>
    Protected Sub SetItinerary(ByVal text As String)
        If textBlock IsNot Nothing Then
            textBlock.Inlines.Clear()

            ' Clean up the itinerary.
            Dim itinerary = CleanElements(text)

            Try
                ' Read the input
                Dim dummyXml = String.Format("<Itinerary xmlns:VirtualEarth=""http://dummy"">{0}</Itinerary>",
                                             itinerary)
                Using stringReader = New StringReader(dummyXml)
                    ' Trace the previous element.
                    Dim previousElement = ""

                    ' Parse the dummy xml.
                    Using xmlReader = System.Xml.XmlReader.Create(stringReader)
                        ' Read each element.
                        Do While xmlReader.Read()
                            ' Add to textblock.
                            If Not String.IsNullOrEmpty(xmlReader.Value) Then
                                If previousElement.StartsWith("VirtualEarth:") Then
                                    textBlock.Inlines.Add(New Run With
                                                          {.Text = xmlReader.Value,
                                                           .FontWeight = FontWeights.Bold})
                                Else
                                    textBlock.Inlines.Add(New Run With {.Text = xmlReader.Value})
                                End If
                            End If

                            ' Store the previous element.
                            If xmlReader.NodeType = XmlNodeType.Element Then
                                previousElement = xmlReader.Name
                            Else
                                previousElement = ""
                            End If
                        Loop
                    End Using
                End Using
            Catch
                ' Strip all xml.
                Dim regex = New Regex("<[/a-zA-Z:]*>", RegexOptions.IgnoreCase Or RegexOptions.Multiline)
                textBlock.Inlines.Add(New Run With {.Text = regex.Replace(itinerary, String.Empty)})
            End Try
        End If
    End Sub
End Class
